import datetime
import os

from sqlalchemy import create_engine, Table, Column, Integer, String, Text, \
    MetaData, DateTime
from sqlalchemy.orm import mapper, sessionmaker


class ClientDatabase:
    """
    Класс для работы с базой данных клиента.
    """

    class KnownUsers:
        """
        Класс - отображение для таблицы всех пользователей.
        """
        username = None

        def __init__(self, user):
            self.id = None
            self.username = user

    class MessageStat:
        """
        Класс - отображение для таблицы статистики переданных сообщений.
        """

        def __init__(self, contact, direction, message):
            self.id = None
            self.contact = contact
            self.direction = direction
            self.message = message
            self.date = datetime.datetime.now()

    class Contacts:
        """
        Класс - отображение для таблицы контактов.
        """
        name = None

        def __init__(self, contact):
            self.id = None
            self.name = contact

    def __init__(self, name):
        path = os.path.dirname(os.path.realpath(__file__))
        filename = f'client_{name}.db3'
        self.database_engine = create_engine(
            f'sqlite:///{os.path.join(path, filename)}',
            echo=False,
            pool_recycle=7200,
            connect_args={
                'check_same_thread': False})

        self.metadata = MetaData()

        users = Table('known_users', self.metadata,
                      Column('id', Integer, primary_key=True),
                      Column('username', String)
                      )

        history = Table('message_history', self.metadata,
                        Column('id', Integer, primary_key=True),
                        Column('contact', String),
                        Column('direction', String),
                        Column('message', Text),
                        Column('date', DateTime)
                        )

        contacts = Table('contacts', self.metadata,
                         Column('id', Integer, primary_key=True),
                         Column('name', String, unique=True)
                         )

        self.metadata.create_all(self.database_engine)

        mapper(self.KnownUsers, users)
        mapper(self.MessageStat, history)
        mapper(self.Contacts, contacts)

        session = sessionmaker(bind=self.database_engine)
        self.session = session()

        self.session.query(self.Contacts).delete()
        self.session.commit()

    def add_contact(self, contact):
        """
        Метод добавляющий контакт в базу данных.
        """
        if not self.session.query(
                self.Contacts).filter_by(
            name=contact).count():
            contact_row = self.Contacts(contact)
            self.session.add(contact_row)
            self.session.commit()

    def contacts_clear(self):
        """
        Метод очищающий таблицу со списком контактов.
        """
        self.session.query(self.Contacts).delete()

    def del_contact(self, contact):
        """
        Метод удаляющий определённый контакт.
        """
        self.session.query(self.Contacts).filter_by(name=contact).delete()

    def add_users(self, users_list):
        """
        Метод заполняющий таблицу известных пользователей.
        """
        self.session.query(self.KnownUsers).delete()
        for user in users_list:
            user_row = self.KnownUsers(user)
            self.session.add(user_row)
        self.session.commit()

    def save_message(self, contact, direction, message):
        """Метод сохраняющий сообщение в базе данных."""
        message_row = self.MessageStat(contact, direction, message)
        self.session.add(message_row)
        self.session.commit()

    def get_contacts(self):
        """Метод возвращающий список всех контактов."""
        return [contact[0]
                for contact in self.session.query(self.Contacts.name).all()]

    def get_users(self):
        """Метод возвращающий список всех известных пользователей."""
        return [user[0]
                for user in self.session.query(self.KnownUsers.username).all()]

    def check_user(self, user):
        """Метод проверяющий существует ли пользователь."""
        if self.session.query(
                self.KnownUsers).filter_by(
            username=user).count():
            return True
        else:
            return False

    def check_contact(self, contact):
        """Метод проверяющий существует ли контакт."""
        if self.session.query(self.Contacts).filter_by(name=contact).count():
            return True
        else:
            return False

    def get_history(self, contact):
        """
        Метод возвращающий историю сообщений с определённым пользователем.
        """
        query = self.session.query(
            self.MessageStat).filter_by(
            contact=contact)
        return [(history_row.contact,
                 history_row.direction,
                 history_row.message,
                 history_row.date) for history_row in query.all()]



import json
from server_dist.server.common.variables import MAX_PACKAGE_LENGTH, ENCODING
from server_dist.server.common.decor import LogDecor


class PortException(Exception):
    def __str__(self):
        return 'Указан некорректный порт'


class ModeException(Exception):
    def __str__(self):
        return 'Указан некорректный режим'


class IncorrectDataReceivedError(Exception):
    def __str__(self):
        return 'Принято некорректное сообщение от удалённого компьютера.'


class NonDictInputError(Exception):
    def __str__(self):
        return 'Аргумент функции должен быть словарём.'


class ServerError(Exception):
    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text


class ReqFieldMissingError(Exception):
    def __init__(self, missing_field):
        self.missing_field = missing_field


@LogDecor()
def get_message(client):
    """
    Функция приёма сообщений от удалённых компьютеров.
    :param client: сокет для передачи данных.
    :return: словарь - сообщение.
    """
    encoded_response = client.recv(MAX_PACKAGE_LENGTH)
    json_response = encoded_response.decode(ENCODING)
    response = json.loads(json_response)
    if isinstance(response, dict):
        return response
    else:
        raise TypeError


@LogDecor()
def send_message(sock, message):
    """
    Функция отправки словарей через сокет.
    :param sock: сокет
    :param message: словарь для передачи
    """
    js_message = json.dumps(message)
    encoded_message = js_message.encode(ENCODING)
    sock.send(encoded_message)



from sys import stderr
from os import path
import logging
from server_dist.server import LOG_LEVEL_FILE, LOG_LEVEL_STREAM

FORMATTER = logging.Formatter(
    '%(asctime)s %(created)f %(levelname)s %(pathname)s %(filename)s '
    '%(lineno)d %(message)s')

PATH_LOG_FILE = path.abspath(path.join(__file__, "../../logs"))
PATH_LOG_FILE = path.join(PATH_LOG_FILE, 'client.log')

STREAM_HANDLER = logging.StreamHandler(stderr)
STREAM_HANDLER.setFormatter(FORMATTER)
STREAM_HANDLER.setLevel(LOG_LEVEL_STREAM)
LOG_FILE = logging.FileHandler(PATH_LOG_FILE, encoding='utf8')
LOG_FILE.setFormatter(FORMATTER)

LOGGER = logging.getLogger('client_logger')
LOGGER.addHandler(STREAM_HANDLER)
LOGGER.addHandler(LOG_FILE)
LOGGER.setLevel(LOG_LEVEL_FILE)

if __name__ == '__main__':
    LOGGER.info('Информационное сообщение')
    LOGGER.debug('Отладочная информация')
    LOGGER.error('Ошибка')
    LOGGER.critical('Критическая ошибка')



from sys import stderr
from os import path
import logging.handlers
from server_dist.server import LOG_LEVEL_FILE, LOG_LEVEL_STREAM

FORMATTER = logging.Formatter(
    '%(asctime)s %(created)f %(levelname)s %(pathname)s %(filename)s '
    '%(lineno)d %(message)s')

PATH_LOG_FILE = path.abspath(path.join(__file__, "../../logs"))
PATH_LOG_FILE = path.join(PATH_LOG_FILE, 'server.log')

STREAM_HND = logging.StreamHandler(stderr)
STREAM_HND.setFormatter(FORMATTER)
STREAM_HND.setLevel(LOG_LEVEL_STREAM)

LOG_FILE = logging.handlers.TimedRotatingFileHandler(PATH_LOG_FILE,
                                                     encoding='utf8',
                                                     interval=1, when='D')
LOG_FILE.setFormatter(FORMATTER)

LOGGER = logging.getLogger('server_logger')
LOGGER.addHandler(STREAM_HND)
LOGGER.addHandler(LOG_FILE)
LOGGER.setLevel(LOG_LEVEL_FILE)

if __name__ == '__main__':
    LOGGER.debug('Отладочная информация')
    LOGGER.info('Информационное сообщение')
    LOGGER.error('Ошибка')
    LOGGER.critical('Критическая ошибка')



import sys
import logging
import traceback
import inspect

if sys.argv[0].find('client') != -1:
    LOGGER = logging.getLogger('client_logger')
else:
    LOGGER = logging.getLogger('server_logger')


class LogDecor:
    def __call__(self, func):
        def log_save(*args, **kwargs):
            ret = func(*args, **kwargs)
            LOGGER.debug(
                f'Была вызвана функция {func.__name__} c '
                f'параметрами {args}, {kwargs}. '
                f'Вызов из модуля {func.__module__}. Вызов из'
                f' функции {traceback.format_stack()[0].strip().split()[-1]}.'
                f'Вызов из функции {inspect.stack()[1][3]}. '
                f'Результат вызова функции {ret}')
            return ret
        return log_save



import logging

DEFAULT_PORT = 6666
DEFAULT_ADDRESS = 'localhost'
MAX_CONNECTIONS = 10
MAX_PACKAGE_LENGTH = 1024
ENCODING = 'utf-8'
SERVER_DATABASE = 'sqlite:///server_base.db3'

PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
GET_CONTACTS = 'get_contacts'
LIST_INFO = 'data_list'
REMOVE_CONTACT = 'remove'
ADD_CONTACT = 'add'
USERS_REQUEST = 'get_users'

ACTION = 'action'
TIME = 'time'
USER = 'user'
ACCOUNT_NAME = 'account_name'
EXIT = 'exit'
SENDER = 'from'
DESTINATION = 'to'
PUBLIC_KEY = 'public_key'
DATA = 'bin'
PUBLIC_KEY_REQUEST = 'public_key_need'

LOG_LEVEL_FILE = logging.DEBUG
LOG_LEVEL_STREAM = logging.ERROR

DEFAULT_MODE = 'send'
DEFAULT_NAME = 'Guest'
MESSAGE = 'message'
MESSAGE_TEXT = 'message_text'

RESPONSE_200 = {RESPONSE: 200}

RESPONSE_400 = {
    RESPONSE: 400,
    ERROR: None
}

RESPONSE_202 = {
    RESPONSE: 202,
    LIST_INFO: None
}

RESPONSE_511 = {
    RESPONSE: 511,
    DATA: None
}

RESPONSE_205 = {
    RESPONSE: 205
}



import dis


class ServerMaker(type):
    def __init__(self, class_name, bases, class_dict):
        methods = []
        attribs = []

        for func_item in class_dict:
            try:
                res = dis.get_instructions(class_dict[func_item])
            except TypeError:
                pass
            else:
                for item in res:
                    if item.opname == 'LOAD_GLOBAL':
                        if item.argval not in methods:
                            methods.append(item.argval)
                    elif item.opname == 'LOAD_ATTR':
                        if item.argval not in attribs:
                            attribs.append(item.argval)

        if 'connect' in methods:
            raise TypeError(
                'Использование метода connect недопустимо в серверном классе')
        if not ('SOCK_STREAM' in attribs and 'AF_INET' in attribs):
            raise TypeError('Некорректная инициализация сокета.')
        super().__init__(class_name, bases, class_dict)


class ClientMaker(type):
    def __init__(self, class_name, bases, class_dict):

        methods = []
        for func in class_dict:
            try:
                ret = dis.get_instructions(class_dict[func])
            except TypeError:
                pass
            else:
                for item in ret:
                    if item.opname == 'LOAD_GLOBAL':
                        if item.argval not in methods:
                            methods.append(item.argval)

        for command in ('accept', 'listen', 'socket'):
            if command in methods:
                raise TypeError(
                    'В классе обнаружено использование запрещённого метода')

        if 'get_message' in methods or 'send_message' in methods:
            pass
        else:
            raise TypeError(
                'Отсутствуют вызовы функций, работающих с сокетами.')
        super().__init__(class_name, bases, class_dict)



from setuptools import setup, find_packages

setup(name="message_client_zda",
      version="0.0.1",
      description="message_client_zda",
      author="Dmitrii Zarubin",
      author_email="zda_87@mail.ru",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )



import argparse
import logging
import os
import sys
from PyQt5.QtWidgets import QApplication, QMessageBox
from client_dist.client.common.utils import ServerError
from client_dist.client.client.client_database import ClientDatabase
from client_dist.client.client.main_window import ClientMainWindow
from client_dist.client.client.start_dialog import UserNameDialog
from Cryptodome.PublicKey import RSA
from client_dist.client.client.transport import ClientTransport
from server_dist.server.common.variables import DEFAULT_PORT, DEFAULT_ADDRESS

CLIENT_LOG = logging.getLogger('client_logger')


def arg_parser():
    """
    Парсер аргументов командной строки.
    Выполняет проверку на корректность номера порта.
    :return: кортеж из 4 элементов адрес сервера, порт, имя пользователя,
    пароль
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('addr', default=DEFAULT_ADDRESS, nargs='?')
    parser.add_argument('port', default=DEFAULT_PORT, type=int, nargs='?')
    parser.add_argument('-n', '--name', default=None, nargs='?')
    parser.add_argument('-p', '--password', default='', nargs='?')
    namespace = parser.parse_args(sys.argv[1:])
    server_address = namespace.addr
    server_port = namespace.port
    client_name = namespace.name
    client_passwd = namespace.password

    if not 1023 < server_port < 65536:
        CLIENT_LOG.critical(
            f'Попытка запуска клиента с неподходящим номером порта: '
            f'{server_port}. Допустимы адреса с 1024 до 65535. '
            f'Клиент завершается.')
        sys.exit(1)

    return server_address, server_port, client_name, client_passwd


if __name__ == '__main__':
    server_address, server_port, client_name, client_passwd = arg_parser()
    CLIENT_LOG.debug('Args loaded')

    client_app = QApplication(sys.argv)

    start_dialog = UserNameDialog()
    if not client_name or not client_passwd:
        client_app.exec_()
        if start_dialog.ok_pressed:
            client_name = start_dialog.client_name.text()
            client_passwd = start_dialog.client_passwd.text()
            CLIENT_LOG.debug(
                f'Using USERNAME = {client_name}, PASSWD = {client_passwd}.')
        else:
            sys.exit(0)

    CLIENT_LOG.info(
        f'Запущен клиент с парамертами: адрес сервера: {server_address}, '
        f'порт: {server_port}, имя пользователя: {client_name}')

    dir_path = os.path.dirname(os.path.realpath(__file__))
    key_file = os.path.join(dir_path, f'{client_name}.key')
    if not os.path.exists(key_file):
        keys = RSA.generate(2048, os.urandom)
        with open(key_file, 'wb') as key:
            key.write(keys.export_key())
    else:
        with open(key_file, 'rb') as key:
            keys = RSA.import_key(key.read())

    CLIENT_LOG.debug("Keys successfully loaded.")
    database = ClientDatabase(client_name)
    try:
        transport = ClientTransport(
            server_port,
            server_address,
            database,
            client_name,
            client_passwd,
            keys)
        CLIENT_LOG.debug("Transport ready.")
    except ServerError as error:
        message = QMessageBox()
        message.critical(start_dialog, 'Ошибка сервера', error.text)
        sys.exit(1)
    transport.setDaemon(True)
    transport.start()
    del start_dialog
    main_window = ClientMainWindow(database, transport, keys)
    main_window.make_connection(transport)
    main_window.setWindowTitle(f'Чат - {client_name}')
    client_app.exec_()
    transport.transport_shutdown()
    transport.join()
